# coding: utf-8
import ast

from transpiler.phases.typing.expr import DUNDER
from transpiler.phases.utils import make_lnd
from transpiler.utils import linenodata


class DesugarCompare(ast.NodeTransformer):
    def visit_Compare(self, node: ast.Compare):
        res = ast.BoolOp(ast.And(), [], **linenodata(node))
        for left, op, right in zip([node.left] + node.comparators, node.ops, node.comparators):
            lnd = make_lnd(left, right)
            if type(op) in (ast.In, ast.NotIn):
                left, right = right, left
            call = ast.Call(
                ast.Attribute(left, f"__{DUNDER[type(op)]}__", **lnd),
                [right],
                [],
                **lnd
            )
            if type(op) == ast.NotIn:
                call = ast.UnaryOp(ast.Not(), call, **lnd)
            res.values.append(call)
        if len(res.values) == 1:
            return res.values[0]
        return res
